/*   $Source: /var/local/cvs/gasnet/template-conduit/gasnet_core_fwd.h,v $
 *     $Date: 2012/08/10 23:04:20 $
 * $Revision: 1.17 $
 * Description: GASNet header for psm conduit core (forward definitions)
 * Copyright (c) 2013-2015 Intel Corporation. All rights reserved.
 * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
 * Terms of use are as specified in license.txt
 */

#ifndef _IN_GASNET_H
#error This file is not meant to be included directly- clients should include gasnet.h
#endif

#ifndef _GASNET_CORE_FWD_H
#define _GASNET_CORE_FWD_H

#define GASNET_CORE_VERSION      0.2
#define GASNET_CORE_VERSION_STR  _STRINGIFY(GASNET_CORE_VERSION)
#define GASNET_CORE_NAME         PSM
#define GASNET_CORE_NAME_STR     _STRINGIFY(GASNET_CORE_NAME)
#define GASNET_CONDUIT_NAME      GASNET_CORE_NAME
#define GASNET_CONDUIT_NAME_STR  _STRINGIFY(GASNET_CONDUIT_NAME)
#define GASNET_CONDUIT_PSM 1

#define GASNETI_MAX_MEDIUM_PSHM 8192
#define GASNETC_MAX_MEDIUM_PSHM 8192

/* GASNET_PSHM defined 1 if this conduit supports PSHM. leave undefined otherwise. */
#if GASNETI_PSHM_ENABLED
#define GASNET_PSHM 1
#endif

/*  defined to be 1 if gasnet_init guarantees that the remote-access memory segment will be aligned  */
/*  at the same virtual address on all nodes. defined to 0 otherwise */
#if GASNETI_DISABLE_ALIGNED_SEGMENTS || GASNET_PSHM
#define GASNET_ALIGNED_SEGMENTS   0 /* user or PSHM disabled segment alignment */
#else
#define GASNET_ALIGNED_SEGMENTS   1
#endif

/* define to 1 if conduit allows internal GASNet fns to issue put/get for remote
   addrs out of segment - not true when PSHM is used */
#if !GASNET_PSHM
#define GASNETI_SUPPORTS_OUTOFSEGMENT_PUTGET 1
#endif

/* conduits should define GASNETI_CONDUIT_THREADS to 1 if they have one or more
   "private" threads which may be used to run AM handlers, even under GASNET_SEQ
   this ensures locking is still done correctly, etc
   */
#if 1
#define GASNETI_CONDUIT_THREADS 1
#endif

/* define to 1 if your conduit may interrupt an application thread
   (e.g. with a signal) to run AM handlers (interrupt-based handler dispatch)
   */
#if 0
#define GASNETC_USE_INTERRUPTS 1
#endif

/* define these to 1 if your conduit supports PSHM, but cannot use the
   default interfaces. (see template-conduit/gasnet_core.c and gasnet_pshm.h)
   */
#if 0
#define GASNETC_GET_HANDLER 1
typedef ### gasnetc_handler_t;
#endif
#if 0
#define GASNETC_TOKEN_CREATE 1
#endif

/* this can be used to add conduit-specific
   statistical collection values (see gasnet_trace.h) */
#define GASNETC_CONDUIT_STATS(CNT,VAL,TIME)

#endif
