# Description: Makefile fragment for psm-conduit, GASNET_PAR mode
# WARNING: This file is automatically generated - do NOT edit directly
# other/fragment-head.mak.  Generated from fragment-head.mak.in by configure.
# Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
# Terms of use are as specified in license.txt

# ----------------------------------------------------------------------
# GASNet client usage instructions:
#
# Clients should include this file in their Makefile, using: (no leading '#')
#     include /usr/shmem/gcc/gasnet-1.24.2-openmpi-hfi/include/psm-conduit/psm-par.mak
# or alternatively, just:
#     include psm-par.mak
# and use a -I/usr/shmem/gcc/gasnet-1.24.2-openmpi-hfi/include/psm-conduit
# command-line option when invoking make
#
# Then in the Makefile, use a compile line something like this:
#  $(GASNET_CC) $(GASNET_CPPFLAGS) $(GASNET_CFLAGS) -c myfile.c
#
# and a link line something like this:
#  $(GASNET_LD) $(GASNET_LDFLAGS) -o myfile myfile.o $(GASNET_LIBS)
# 
# For GASNet clients who need finer-grained control over C compiler options, 
#  GASNET_CFLAGS is an alias for: 
#     $(GASNET_OPT_CFLAGS) $(GASNET_MISC_CFLAGS)
#  GASNET_CPPFLAGS is an alias for: 
#     $(GASNET_MISC_CPPFLAGS) $(GASNET_DEFINES) $(GASNET_INCLUDES)
# and those finer-grained variables can be used rather than the convenience aliases
#
# ----------------------------------------------------------------------

# ----------------------------------------------------------------------
# GASNet conduit-writer instructions:
#
# Most conduit-specific tweaks for these fragments can be accomplished 
# by setting specially-named variables in your conduit.mak file:
#
# * Variables of the form CONDUIT_WHATEVER will be added to the GASNET_WHATEVER
#   variable in the fragment.
#
# * Variables of the form CONDUIT_WHATEVER_<MODE> (where <MODE> is SEQ, PAR or PARSYNC)
#   will be added to the GASNET_WHATEVER variable for the <MODE> fragment only.
#
# * Settings which are contingent on whether the fragment is being used 
#   within the build tree or from an installed copy should be controlled using
#   the standard ###NO INSTALL### marker (without the internal space)
#
# * In the rare cases where a conduit needs to completely replace the 
#   default value for a GASNET_WHATEVER variable (rather than simply adding
#   to it), this can be accomplished by setting one of the GASNET_WHATEVER_OVERRIDE
#   variables in the conduit.mak file. Note this is only supported for the 
#   overridable variables listed below, and should only be used as a last resort
#   because it clobbers the default values established by configure and
#   complicates GASNet framework maintenance.
#
# ----------------------------------------------------------------------

# ----------------------------------------------------------------------
# conduit-overridable variables 

GASNET_LD_OVERRIDE = /usr/bin/gcc 
GASNET_LDFLAGS_OVERRIDE = -O3 --param max-inline-insns-single=35000 --param inline-unit-growth=10000 --param large-function-growth=200000 -Winline 

# ----------------------------------------------------------------------
# psm-conduit/conduit.mak settings
# Copyright (c) 2014-2015 Intel Corporation. All rights reserved.

# When psm-conduit uses an MPI-based bootstrapper, we must
# link using the system MPI compiler
GASNET_LD_OVERRIDE = /usr/mpi/gcc/openmpi-1.10.4-hfi/bin/mpicc 
GASNET_LDFLAGS_OVERRIDE =  -O3 --param max-inline-insns-single=35000 --param inline-unit-growth=10000 --param large-function-growth=200000 -Winline 
MPI_COMPAT_LIBS = 
# Some platforms need extra -libs for the socket calls in ssh-spawner:
SSH_LIBS = 
CONDUIT_LDFLAGS = -L/usr/lib64  
CONDUIT_LIBS = -pthread -lpsm2 $(MPI_COMPAT_LIBS) $(SSH_LIBS)  
# $Source: bitbucket.org:berkeleylab/gasnet.git/other/fragment-body.mak.in $
# ----------------------------------------------------------------------
# Following section other/fragment-body.mak.  Generated from fragment-body.mak.in by configure.

# ----------------------------------------------------------------------
# Directory-based options

GASNET_INCLUDES =  -I/usr/shmem/gcc/gasnet-1.24.2-openmpi-hfi/include -I/usr/shmem/gcc/gasnet-1.24.2-openmpi-hfi/include/psm-conduit $(CONDUIT_INCLUDES) $(CONDUIT_INCLUDES_PAR)
GASNET_LIBDIRS = -L/usr/shmem/gcc/gasnet-1.24.2-openmpi-hfi/lib $(CONDUIT_LIBDIRS) $(CONDUIT_LIBDIRS_PAR)


# ----------------------------------------------------------------------
# C compiler and options

GASNET_CC = /usr/bin/gcc 

GASNET_OPT_CFLAGS = -O3 --param max-inline-insns-single=35000 --param inline-unit-growth=10000 --param large-function-growth=200000 -Winline $(CONDUIT_OPT_CFLAGS) $(CONDUIT_OPT_CFLAGS_PAR)
GASNET_MISC_CFLAGS =  $(CONDUIT_MISC_CFLAGS) $(CONDUIT_MISC_CFLAGS_PAR)
GASNET_MISC_CPPFLAGS =  $(CONDUIT_MISC_CPPFLAGS) $(CONDUIT_MISC_CPPFLAGS_PAR)

GASNET_EXTRADEFINES_SEQ = 
GASNET_EXTRADEFINES_PAR = -D_REENTRANT -D_GNU_SOURCE
GASNET_EXTRADEFINES_PARSYNC = -D_REENTRANT -D_GNU_SOURCE

GASNET_DEFINES = -DGASNET_PAR $(GASNET_EXTRADEFINES_PAR) $(CONDUIT_DEFINES) $(CONDUIT_DEFINES_PAR)

# ----------------------------------------------------------------------
# Documented compilation convenience aliases

GASNET_CFLAGS = $(GASNET_OPT_CFLAGS) $(GASNET_MISC_CFLAGS) $(MANUAL_CFLAGS)
GASNET_CPPFLAGS = $(GASNET_MISC_CPPFLAGS) $(GASNET_DEFINES) $(GASNET_INCLUDES)

# ----------------------------------------------------------------------
# C++ compiler

GASNET_CXX = /usr/bin/g++ 
GASNET_CXXFLAGS = -O2  $(MANUAL_CXXFLAGS)
GASNET_CXXCPPFLAGS =   $(GASNET_DEFINES) $(GASNET_INCLUDES)

# ----------------------------------------------------------------------
# linker and options

GASNET_LD = $(GASNET_LD_OVERRIDE)

# linker flags that GASNet clients should use 
GASNET_LDFLAGS = $(GASNET_LDFLAGS_OVERRIDE) $(CONDUIT_LDFLAGS) $(CONDUIT_LDFLAGS_PAR) $(MANUAL_LDFLAGS)

GASNET_EXTRALIBS_SEQ = 
GASNET_EXTRALIBS_PAR = -lpthread
GASNET_EXTRALIBS_PARSYNC = -lpthread

# libraries that GASNet clients should append to link line
GASNET_LIBS =                             \
    $(GASNET_LIBDIRS)                     \
    -lgasnet-psm-par \
    $(CONDUIT_LIBS)                       \
    $(CONDUIT_LIBS_PAR)        \
    $(GASNET_EXTRALIBS_PAR)    \
                        \
    -L/usr/lib/gcc/x86_64-redhat-linux/4.8.5 -lgcc                              \
                                    \
    -lm                                \
    $(MANUAL_LIBS)

# ----------------------------------------------------------------------
